@@ -1,4 +1,4 @@
-/* $Id: SUPDrvIOC.h 101422 2015-07-03 12:10:49Z bird $ */
+/* $Id: SUPDrvIOC.h 101982 2015-08-06 23:34:04Z bird $ */
 /** @file
  * VirtualBox Support Driver - IOCtl definitions.
  */
@@ -214,7 +214,7 @@ typedef SUPREQHDR *PSUPREQHDR;
  * @todo Pending work on next major version change:
  *          - nothing.
  */
-#define SUPDRV_IOC_VERSION                              0x00230001
+#define SUPDRV_IOC_VERSION                              0x00230003
 
 /** SUP_IOCTL_COOKIE. */
 typedef struct SUPCOOKIE
@@ -1522,6 +1522,7 @@ AssertCompileMemberAlignment(SUPTSCDELTAMEASURE, u, 8);
 AssertCompileSize(SUPTSCDELTAMEASURE, 6*4 + 4+1+1+1+1);
 /** @} */
 
+
 /** @name SUP_IOCTL_TSC_READ
  * Reads the TSC and apply TSC-delta if applicable, determining the delta if
  * necessary (i64TSCDelta = INT64_MAX).
@@ -1558,6 +1559,33 @@ AssertCompileMemberAlignment(SUPTSCREAD, u, 8);
 AssertCompileSize(SUPTSCREAD, 6*4 + 2*8);
 /** @} */
 
+
+/** @name SUP_IOCTL_GIP_SET_FLAGS
+ * Set GIP flags.
+ *
+ * @{
+ */
+#define SUP_IOCTL_GIP_SET_FLAGS                         SUP_CTL_CODE_SIZE(39, SUP_IOCTL_GIP_SET_FLAGS_SIZE)
+#define SUP_IOCTL_GIP_SET_FLAGS_SIZE                    sizeof(SUPGIPSETFLAGS)
+#define SUP_IOCTL_GIP_SET_FLAGS_SIZE_IN                 sizeof(SUPGIPSETFLAGS)
+#define SUP_IOCTL_GIP_SET_FLAGS_SIZE_OUT                sizeof(SUPREQHDR)
+typedef struct SUPGIPSETFLAGS
+{
+    /** The header. */
+    SUPREQHDR               Hdr;
+    union
+    {
+        struct
+        {
+            /** The AND flags mask, see SUPGIP_FLAGS_XXX. */
+            uint32_t        fAndMask;
+            /** The OR flags mask, see SUPGIP_FLAGS_XXX. */
+            uint32_t        fOrMask;
+        } In;
+    } u;
+} SUPGIPSETFLAGS, *PSUPGIPSETFLAGS;
+/** @} */
+
 #pragma pack()                          /* paranoia */
 
 #endif
