@@ -1,10 +1,10 @@
-/* $Id: SUPDrvTracer.cpp 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: SUPDrvTracer.cpp 100877 2015-06-09 14:23:56Z bird $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Tracer Interface.
  */
 
 /*
- * Copyright (C) 2012-2013 Oracle Corporation
+ * Copyright (C) 2012-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -339,13 +339,16 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
                         u64Tmp, pVtgHdr->uProbeLocs.u64, pVtgHdr->uProbeLocsEnd.u64);
             return VERR_SUPDRV_VTG_BAD_HDR_TOO_MUCH;
         }
+        /*SUPR0Printf("supdrvVtgValidateHdr: cbProbeLocs %#x -> %#x\n", pVtgHdr->cbProbeLocs, (uint32_t)u64Tmp);*/
         pVtgHdr->cbProbeLocs  = (uint32_t)u64Tmp;
 
         u64Tmp = pVtgHdr->uProbeLocs.u64 - uVtgHdrAddr;
 #ifdef RT_OS_DARWIN
         /* The loader and/or ld64-97.17 seems not to generate fixups for our
            __VTGObj section. Detect this by comparing them with the
-           u64VtgObjSectionStart member and assume max image size of 4MB. */
+           u64VtgObjSectionStart member and assume max image size of 4MB.
+           Seems to be worked around by the __VTGPrLc.End and __VTGPrLc.Begin
+           padding fudge, meaning that the linker misplaced the relocations. */
         if (   (int64_t)u64Tmp != (int32_t)u64Tmp
             && pVtgHdr->u64VtgObjSectionStart != uVtgHdrAddr
             && pVtgHdr->u64VtgObjSectionStart < _4M
@@ -353,6 +356,7 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
             && !fUmod)
         {
             uint64_t offDelta = uVtgHdrAddr - pVtgHdr->u64VtgObjSectionStart;
+            /*SUPR0Printf("supdrvVtgValidateHdr: offDelta=%#llx\n", offDelta);*/
             pVtgHdr->uProbeLocs.u64        += offDelta;
             pVtgHdr->uProbeLocsEnd.u64     += offDelta;
             u64Tmp += offDelta;
@@ -364,6 +368,7 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
                         u64Tmp, pVtgHdr->uProbeLocs.u64, uVtgHdrAddr);
             return VERR_SUPDRV_VTG_BAD_HDR_PTR;
         }
+        /*SUPR0Printf("supdrvVtgValidateHdr: offProbeLocs %#x -> %#x\n", pVtgHdr->offProbeLocs, (int32_t)u64Tmp);*/
         pVtgHdr->offProbeLocs = (int32_t)u64Tmp;
     }
 
@@ -371,7 +376,10 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
      * The non-area description fields.
      */
     if (memcmp(pVtgHdr->szMagic, VTGOBJHDR_MAGIC, sizeof(pVtgHdr->szMagic)))
+    {
+        SUPR0Printf("supdrvVtgValidateHdr: %p: %.16Rhxs\n", pVtgHdr, pVtgHdr->szMagic);
         return VERR_SUPDRV_VTG_MAGIC;
+    }
     if (   pVtgHdr->cBits != ARCH_BITS
         && (   !fUmod
             || (   pVtgHdr->cBits != 32
@@ -1470,13 +1478,13 @@ SUPR0TracerFireProbe:                                                   \n\
 ");
 # if   defined(RT_ARCH_AMD64)
 __asm__(" \
-	    movq    g_pfnSupdrvProbeFireKernel(%rip), %rax                  \n\
-	    jmp	    *%rax \n\
+            movq    g_pfnSupdrvProbeFireKernel(%rip), %rax                  \n\
+            jmp     *%rax \n\
 ");
 # elif defined(RT_ARCH_X86)
 __asm__("\
-	    movl    g_pfnSupdrvProbeFireKernel, %eax                        \n\
-	    jmp	    *%eax \n\
+            movl    g_pfnSupdrvProbeFireKernel, %eax                        \n\
+            jmp     *%eax \n\
 ");
 # else
 #  error "Which arch is this?"
@@ -2136,6 +2144,7 @@ int  VBOXCALL   supdrvIOCtl_TracerUmodDeregister(PSUPDRVDEVEXT pDevExt, PSUPDRVS
         RTR0MemObjFree(pUmod->hMemObjMap, false /*fFreeMappings*/);
         RTR0MemObjFree(pUmod->hMemObjLock, false /*fFreeMappings*/);
         RTMemFree(pUmod);
+        rc = VINF_SUCCESS;
     }
     else
         rc = VERR_NOT_FOUND;
