@@ -1,4 +1,4 @@
-/* $Id: SUPLibAll.cpp 99558 2015-04-14 13:02:36Z bird $ */
+/* $Id: SUPLibAll.cpp 101971 2015-08-06 14:53:27Z ramshankar $ */
 /** @file
  * VirtualBox Support Library - All Contexts Code.
  */
@@ -269,5 +269,62 @@ SUPDECL(uint64_t) SUPGetCpuHzFromGipForAsyncMode(PSUPGLOBALINFOPAGE pGip)
 }
 
 
+/**
+ * Worker for SUPIsTscFreqCompatible().
+ *
+ * @returns true if it's compatible, false otherwise.
+ * @param   uBaseCpuHz      The reference CPU frequency of the system.
+ * @param   uCpuHz          The CPU frequency to compare with the base.
+ * @param   fRelax          Whether to use a more relaxed threshold (like
+ *                          for when running in a virtualized environment).
+ *
+ * @remarks Don't use directly, use SUPIsTscFreqCompatible() instead. This is
+ *          to be used by tstGIP-2 or the like.
+ */
+SUPDECL(bool) SUPIsTscFreqCompatibleEx(uint64_t uBaseCpuHz, uint64_t uCpuHz, bool fRelax)
+{
+    if (uBaseCpuHz != uCpuHz)
+    {
+        /* Arbitrary tolerance threshold, tweak later if required, perhaps
+           more tolerance on lower frequencies and less tolerance on higher. */
+        uint16_t uFact = !fRelax ? 666 /* 0.15% */ : 125 /* 0.8% */;
+        uint64_t uThr  = uBaseCpuHz / uFact;
+        uint64_t uLo   = uBaseCpuHz - uThr;
+        uint64_t uHi   = uBaseCpuHz + uThr;
+        if (   uCpuHz < uLo
+            || uCpuHz > uHi)
+            return false;
+    }
+    return true;
+}
+
+
+/**
+ * Checks if the provided TSC frequency is close enough to the computed TSC
+ * frequency of the host.
+ *
+ * @returns true if it's compatible, false otherwise.
+ * @param   uCpuHz          The TSC frequency to check.
+ * @param   puGipCpuHz      Where to store the GIP TSC frequency used
+ *                          during the compatibility test - optional.
+ * @param   fRelax          Whether to use a more relaxed threshold (like
+ *                          for when running in a virtualized environment).
+ */
+SUPDECL(bool) SUPIsTscFreqCompatible(uint64_t uCpuHz, uint64_t *puGipCpuHz, bool fRelax)
+{
+    PSUPGLOBALINFOPAGE pGip = g_pSUPGlobalInfoPage;
+    bool     fCompat = false;
+    uint64_t uGipCpuHz = 0;
+    if (   pGip
+        && pGip->u32Mode != SUPGIPMODE_ASYNC_TSC)
+    {
+        uGipCpuHz = pGip->u64CpuHz;
+        fCompat   = SUPIsTscFreqCompatibleEx(uGipCpuHz, uCpuHz, fRelax);
+    }
+    if (puGipCpuHz)
+        *puGipCpuHz = uGipCpuHz;
+    return fCompat;
+}
+
 #endif /* RT_ARCH_AMD64 || RT_ARCH_X86 */
 
