@@ -1,4 +1,4 @@
-/* $Id: threadctxhooks-r0drv-linux.c 100485 2015-05-20 16:39:40Z ramshankar $ */
+/* $Id: threadctxhooks-r0drv-linux.c 102031 2015-08-11 14:39:19Z bird $ */
 /** @file
  * IPRT - Thread Context Switching Hook, Ring-0 Driver, Linux.
  */
@@ -166,6 +166,8 @@ DECLINLINE(void) rtThreadCtxHookDisable(PRTTHREADCTXHOOKINT pThis)
 
 RTDECL(int) RTThreadCtxHookCreate(PRTTHREADCTXHOOK phCtxHook, uint32_t fFlags, PFNRTTHREADCTXHOOK pfnCallback, void *pvUser)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
     /*
      * Validate input.
      */
@@ -180,7 +182,10 @@ RTDECL(int) RTThreadCtxHookCreate(PRTTHREADCTXHOOK phCtxHook, uint32_t fFlags, P
      */
     pThis = (PRTTHREADCTXHOOKINT)RTMemAllocZ(sizeof(*pThis));
     if (RT_UNLIKELY(!pThis))
+    {
+        IPRT_LINUX_RESTORE_EFL_AC();
         return VERR_NO_MEMORY;
+    }
     pThis->u32Magic     = RTTHREADCTXHOOKINT_MAGIC;
     pThis->hOwner       = RTThreadNativeSelf();
     pThis->fEnabled     = false;
@@ -190,7 +195,12 @@ RTDECL(int) RTThreadCtxHookCreate(PRTTHREADCTXHOOK phCtxHook, uint32_t fFlags, P
     pThis->PreemptOps.sched_out = rtThreadCtxHooksLnxSchedOut;
     pThis->PreemptOps.sched_in  = rtThreadCtxHooksLnxSchedIn;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 2, 0)
+    preempt_notifier_inc();
+#endif
+
     *phCtxHook = pThis;
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTThreadCtxHookCreate);
@@ -198,6 +208,8 @@ RT_EXPORT_SYMBOL(RTThreadCtxHookCreate);
 
 RTDECL(int ) RTThreadCtxHookDestroy(RTTHREADCTXHOOK hCtxHook)
 {
+    IPRT_LINUX_SAVE_EFL_AC();
+
     /*
      * Validate input.
      */
@@ -220,9 +232,14 @@ RTDECL(int ) RTThreadCtxHookDestroy(RTTHREADCTXHOOK hCtxHook)
         Assert(!pThis->fEnabled); /* paranoia */
     }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 2, 0)
+    preempt_notifier_dec();
+#endif
+
     ASMAtomicWriteU32(&pThis->u32Magic, ~RTTHREADCTXHOOKINT_MAGIC);
     RTMemFree(pThis);
 
+    IPRT_LINUX_RESTORE_EFL_AC();
     return VINF_SUCCESS;
 }
 RT_EXPORT_SYMBOL(RTThreadCtxHookDestroy);
@@ -241,6 +258,7 @@ RTDECL(int) RTThreadCtxHookEnable(RTTHREADCTXHOOK hCtxHook)
     Assert(!pThis->fEnabled);
     if (!pThis->fEnabled)
     {
+        IPRT_LINUX_SAVE_EFL_AC();
         Assert(pThis->PreemptOps.sched_out == rtThreadCtxHooksLnxSchedOut);
         Assert(pThis->PreemptOps.sched_in == rtThreadCtxHooksLnxSchedIn);
 
@@ -251,6 +269,8 @@ RTDECL(int) RTThreadCtxHookEnable(RTTHREADCTXHOOK hCtxHook)
         pThis->fEnabled = true;
         preempt_notifier_register(&pThis->LnxPreemptNotifier);
         preempt_enable();
+
+        IPRT_LINUX_RESTORE_EFL_AC();
     }
 
     return VINF_SUCCESS;
@@ -275,7 +295,11 @@ RTDECL(int) RTThreadCtxHookDisable(RTTHREADCTXHOOK hCtxHook)
          * Deregister the callback.
          */
         if (pThis->fEnabled)
+        {
+            IPRT_LINUX_SAVE_EFL_AC();
             rtThreadCtxHookDisable(pThis);
+            IPRT_LINUX_RESTORE_EFL_AC();
+        }
     }
     return VINF_SUCCESS;
 }
